unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, Buttons, ExtCtrls, Menus, ClipBrd, Dialogs, iCoolKey, iClip;

type
	TClipMode = (cmStop, cmRecord, cmPlay);
const
    CLIP_MODE_TEXT: array[TClipMode] of String
        = ('NOW STOPPING...', 'NOW RECORDING...', 'NOW PLAYING...');
    CLIP_MODE_COLOR: array[TClipMode] of TColor
        = (clGray, clRed, clBlue);
type
  TMainForm = class(TForm)
    pnlState: TPanel;
    MainMenu: TMainMenu;
    MenuFile: TMenuItem;
    MenuClip: TMenuItem;
    MenuExit: TMenuItem;
    PLAY1: TMenuItem;
    MenuPlay: TMenuItem;
    MenuRecord: TMenuItem;
    MenuStop: TMenuItem;
    MenuHelp: TMenuItem;
    MenuVersion: TMenuItem;
    MenuClear: TMenuItem;
    MenuSave: TMenuItem;
    SaveDialog: TSaveDialog;
    OpenDialog: TOpenDialog;
    MenuOpen: TMenuItem;
    N1: TMenuItem;
    MenuCRLF: TMenuItem;
    N2: TMenuItem;
    Panel1: TPanel;
    SP3: TSpeedButton;
    SP4: TSpeedButton;
    SP5: TSpeedButton;
    SP6: TSpeedButton;
    MemClip: TMemo;
    Bevel1: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MenuRecordClick(Sender: TObject);
    procedure MenuPlayClick(Sender: TObject);
    procedure MenuStopClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuVersionClick(Sender: TObject);
    procedure MenuClearClick(Sender: TObject);
    procedure MenuSaveClick(Sender: TObject);
    procedure MenuOpenClick(Sender: TObject);
  private
    FClipMode   : TClipMode;
    FClipWatcher: TClipWatcher;
    FPasteWatcher: TPasteWatcher;
    FCoolKeys: TCoolKeys;
    FDataPath: String;
    FConfPath: String;
  private
    procedure OnCopyText();
    procedure OnPasteText();
    procedure RegistHotKey();
    procedure ClipRecord();
    procedure ClipPlay();
    procedure ClipStop();
  end;

var
  MainForm: TMainForm;

implementation

uses
    Info;

{$R *.dfm}

//-----------------------------------------------------------------------------
//  zbgL[o^
procedure TMainForm.RegistHotKey();
var
    LModifiers: TShiftState;
    LVirtkey: Word;
begin
    FCoolKeys.Clear();
    //  L^[hp
    ShortCutToKey(MenuRecord.ShortCut, LVirtKey, LModifiers);
    if FCoolKeys.Add(ClipRecord, LModifiers, LVirtkey) = -1 then MenuRecord.ShortCut := 0;
    //  Đ[hp
    ShortCutToKey(MenuPlay.ShortCut, LVirtKey, LModifiers);
    if FCoolKeys.Add(ClipPlay, LModifiers, LVirtkey) = -1 then MenuPlay.ShortCut := 0;
    //  ~p
    ShortCutToKey(MenuStop.ShortCut, LVirtKey, LModifiers);
    if FCoolKeys.Add(ClipStop, LModifiers, LVirtkey) = -1 then MenuStop.ShortCut := 0;
end;

//-----------------------------------------------------------------------------
//  Rs[o
procedure TMainForm.OnCopyText();
begin
    //  ŋL^[h烊Xgɒǉ
    if Clipboard.HasFormat(CF_TEXT) and (FClipMode = cmRecord) then
        MemClip.Lines.Add(ClipBoard.AsText);
    ;
end;

//-----------------------------------------------------------------------------
//  y[Xgo
procedure TMainForm.OnPasteText();
var
    LLine: Integer;
    LText: String;
    Data: THandle;
    Buff: Pointer;
begin
	inherited;
    //  Đ[hȂ烊Xg珇ɃNbv{[hɑ荞
    if (FClipMode = cmPlay) then
    begin
        if (MemClip.Lines.Count > 0) then
        begin
            //  ݍs擾
            LLine := SendMessage(MemClip.Handle, EM_LINEFROMCHAR, MemClip.SelStart, 0);
            LText := MemClip.Lines[LLine];

            //  ̍sֈړ
            if (LLine < MemClip.Lines.Count - 1) then Inc(LLine) else LLine := 0;
            MemClip.SelStart := SendMessage(MemClip.Handle, EM_LINEINDEX, LLine, 0);
            MemClip.Perform(EM_SCROLLCARET, 0, 0);
        end
        else
        if (Clipboard.HasFormat(CF_TEXT)) then LText := ClipBoard.AsText
        else LText := ''
        ;

        //  sǉ
        if MenuCRLF.Checked then LText := LText + #13#10;

        //  荞
        Data := GlobalAlloc(GMEM_MOVEABLE, Length(LText) + 1);
        Buff := GlobalLock(Data);
        StrCopy(Buff, PChar(LText));
        SetClipboardData(CF_TEXT, Data);
        GlobalUnlock(Data);
    end;
end;

//-----------------------------------------------------------------------------
//  FormCreate
procedure TMainForm.FormCreate(Sender: TObject);
begin
    inherited;
    // ^CgZbg
    SetFormCaption(Self);
    //  ݒǂݍ
    FConfPath := ChangeFileExt(ParamStr(0), '.ini');
    LoadPropertyFromFile(FConfPath, Self);
    //  Õf[^Γǂݍ
    FDataPath := ChangeFileExt(ParamStr(0), '.txt');
    if (FileExists(FDataPath)) then MemClip.Lines.LoadFromFile(FDataPath);
    // Rs[oNX
    FClipWatcher := TClipWatcher.Create(OnCopyText);
    // y[XgoNX
    FPasteWatcher := TPasteWatcher.Create(OnPasteText);
    //  zbgL[o^NX
    FCoolKeys := TCoolKeys.Create();
    RegistHotKey();
    // [h
    FClipMode := cmStop;
    // ~[hɐݒ
    ClipStop();
end;

//-----------------------------------------------------------------------------
//  FormDestroy
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    //  ̃f[^ۑ
    MemClip.Lines.SaveToFile(FDataPath);
    //  ݒۑ
    SavePropertyToFile(FConfPath, Self);
    // ~
    ClipStop();
    // CX^X
    FClipWatcher.Free();
    FPasteWatcher.Free();
    FCoolKeys.Free();
    inherited;
end;

//-----------------------------------------------------------------------------
// L^[hɕύX
procedure TMainForm.ClipRecord();
begin
    FClipMode := cmRecord;
    pnlState.Caption := CLIP_MODE_TEXT[FClipMode];
    pnlState.Color := CLIP_MODE_COLOR[FClipMode];
end;

//-----------------------------------------------------------------------------
procedure TMainForm.MenuRecordClick(Sender: TObject);
begin
    ClipRecord();
end;

//-----------------------------------------------------------------------------
//  Đ[hɕύX
procedure TMainForm.ClipPlay();
begin
    FClipMode := cmPlay;
    pnlState.Caption := CLIP_MODE_TEXT[FClipMode];
    pnlState.Color := CLIP_MODE_COLOR[FClipMode];
    FPasteWatcher.SetRender();
end;

//-----------------------------------------------------------------------------
procedure TMainForm.MenuPlayClick(Sender: TObject);
begin
    ClipPlay();
end;

//-----------------------------------------------------------------------------
//  ~
procedure TMainForm.ClipStop();
begin
    FClipMode := cmStop;
    pnlState.Caption := CLIP_MODE_TEXT[FClipMode];
    pnlState.Color := CLIP_MODE_COLOR[FClipMode];
end;

//-----------------------------------------------------------------------------
procedure TMainForm.MenuStopClick(Sender: TObject);
begin
    ClipStop();
end;

//-----------------------------------------------------------------------------
//  I
procedure TMainForm.MenuExitClick(Sender: TObject);
begin
    Self.Close;
end;

//-----------------------------------------------------------------------------
//  o[W
procedure TMainForm.MenuVersionClick(Sender: TObject);
begin
    OpenInfoForm();
end;

//-----------------------------------------------------------------------------
//  XgNA
procedure TMainForm.MenuClearClick(Sender: TObject);
begin
    MemClip.Clear();
end;

//-----------------------------------------------------------------------------
//  Xgǂݍ
procedure TMainForm.MenuOpenClick(Sender: TObject);
begin
    if OpenDialog.Execute() then MemClip.Lines.LoadFromFile(OpenDialog.FileName);
end;

//-----------------------------------------------------------------------------
//  Xgۑ
procedure TMainForm.MenuSaveClick(Sender: TObject);
begin
    if SaveDialog.Execute() then MemClip.Lines.SaveToFile(SaveDialog.FileName);
end;

end.
